package org.amse.mARICa.autoTests;

import org.amse.mARICa.Logger;
import org.amse.mARICa.game.*;
import org.amse.mARICa.model.*;
import org.amse.mARICa.model.impl.*;
import org.amse.mARICa.model.impl.players.*;

import junit.framework.TestCase;

public class MyTests extends TestCase {
	
	private boolean equalsArr(int [] a, int [] b){
		if (a == b){
			return true;
		}
		if (a.length != b.length){
			return false;
		}
		for (int i = 0; i < a.length; i++){
			if (a[i] != b[i]){
				return false;
			}
		}
		return true;
	}
	/**
	 *    IBoard
	 */
	public void Qtest1Board(){
		IBoard board = new Board(6, 4);
		assertTrue("Verify: getCountPots()", board.getCountPots() == 6);
		assertTrue("Verify: getCountBeans()", board.getCountBeans() == 4);
		assertTrue("Verify: getCountInPot() in Mancala1", board.getCountInPot(6) == 0);
		assertTrue("Verify: getCountInPot() in Mancala2", board.getCountInPot(13) == 0);
		assertTrue("Verify: getCountInPot() in Pot1", board.getCountInPot(1) == 4);
		assertTrue("Verify: getCountInPot() in Pot10", board.getCountInPot(10) == 4);
		assertFalse("Verify: getCountInPot() in notExist", board.getCountInPot(14) == 0);		
		assertTrue("Verify: canChoose() pot 2", board.canChoose(ESeatPlayer.DOWN, 2) );
		assertFalse("Verify: canChoose() pot Mancala1", board.canChoose(ESeatPlayer.DOWN, 6) );
		assertTrue("Verify: canChoose() pot 2", board.canChoose(ESeatPlayer.UP, 2) );
		assertFalse("Verify: canChoose() pot Mancala2", board.canChoose(ESeatPlayer.UP, 13) );
		int [] ar1 = new int [] {4,4,4,4,4,4,0,4,4,4,4,4,4,0}; 
		assertTrue("Verify: state Board", equalsArr( board.getState(),ar1));	
		board.setCountBeans(2);
		int [] ar2 = new int [] {2,2,2,2,2,2,0,2,2,2,2,2,2,0}; 
		assertTrue("Verify: state Board", equalsArr( board.getState(),ar2));
		
	}
	
	public void Qtest4AIPlayer(){
		IBoard board = new Board(4, 4);		
		Game game = new Game(board);		
		LocalPlayer player1 = new LocalPlayer(EKindPlayer.HUMANLOC, game);
		IPlayer player2 = new AIPlayer(EKindPlayer.COMP, game);
		game.setPlayer1(player1);
		game.setPlayer2(player2);
		((AIPlayer)player2).setRandomChoose(true);
		
		try {
			player1.getMove();		
			player1.setPot(0);
			if (!player1.isReady()){}
			int pot = player1.getChoose();
			boolean isAgainMove = board.choose(player1.getSeat(), pot);
			
			if (isAgainMove){
				player1.getMove();
				player1.setPot(1);
				if (!player1.isReady()){}
				pot = player1.getChoose();
				isAgainMove = board.choose(player1.getSeat(), pot);
			}else{
				assertFalse("Verify: isAgainMove", true);
			}
			int [] ar = new int [] {0,0,6,6,2,5,5,4,4,0}; 
			assertTrue("Verify: state Board", equalsArr( board.getState(),ar));
			if (!isAgainMove){
				player2.getMove();
				if (!player1.isReady()){}
				pot = player1.getChoose();
				isAgainMove = board.choose(player1.getSeat(), pot);
			}
		} catch (BantumiException e) {
			assertFalse("Verify: BantumiException", true);
		}	
	}
	
	/*public void test5AIPlayer(){
		System.out.println("Test2:");
		
		IBoard board = new Board(4, 3);		
		Game game = new Game(board);
		IPlayer player1 = new LocalPlayer(EKindPlayer.HUMANLOC, game);
		IPlayer player2 = new AIPlayer(EKindPlayer.COMP, game);
		game.setPlayer1(player1);
		game.setPlayer2(player2);
		((AIPlayer)player2).setRandomChoose(true);
		game.start();
		doPlayerDownMove(board, 0);
		doAiUpMove(board, player2);	
		doAiUpMove(board, player2);
	}*/
	
	
	public void Qtest6RunningGame(){
		IBoard board = new Board(4, 3);		
		Game game = new Game(board);
		AIPlayer player1 = new AIPlayer(EKindPlayer.COMP, game, 2, false);
		AIPlayer player2 = new AIPlayer(EKindPlayer.COMP, game, 6, false);				
		game.setPlayer1(player1);
		game.setPlayer2(player2);				
		game.start();		
		while(!game.isWin());
		assertTrue("Verify: process of game", game.getWiner() == EKindWins.WINDOWN);
		Logger.getInctanse().setLog(true);
		board.logBoard();		
	}
	
	/**
	 *    IPlayer
	 */
	public void Qtest2Player(){
		IPlayer pl = new LocalPlayer(EKindPlayer.HUMANLOC, null);
		assertFalse("Verify: isCurent()", pl.isCurent());
		pl.setCurrent(true);
		assertTrue("Verify: isCurent()", pl.isCurent());
		assertTrue("Verify: getKind()", pl.getKindPlayer() == EKindPlayer.HUMANLOC);
		assertFalse("Verify: getKind()", pl.getKindPlayer() == EKindPlayer.HUMANLAN);
		assertFalse("Verify: getKind()", pl.getKindPlayer() == EKindPlayer.COMP);
		assertFalse("Verify: isReady()", pl.isReady());
		pl.setReady(true);
		assertTrue("Verify: isReady()", pl.isReady());
		pl.setSeat(ESeatPlayer.DOWN);
		assertTrue("Verify: getSeat()", pl.getSeat() == ESeatPlayer.DOWN);
		try {
			pl.setStopGame(true);
		} catch (BantumiException e) {
			assertFalse("Verify: BantumiException", true);
		}
		assertTrue("Verify: getStopGame()", pl.getStopGame());
		assertTrue("Verify: getChoose()", pl.getChoose() < 0);
	}
	/**
	 *    IGame
	 *      12  11  10  9  8  7 
	 * 13-  |   |   |   |  |  | -6
	 *      0   1   2   3  4  5
	 */
	@SuppressWarnings("unused")
	private void forTest3Game(){
		IBoard board = new Board(6,4);
		IGame game = new Game(board);
		IPlayer pl1 = new LocalPlayer(EKindPlayer.HUMANLOC, game);
		IPlayer pl2 = new LocalPlayer(EKindPlayer.COMP, game);	
		game.setPlayer1(pl1);
		game.setPlayer2(pl2);
		int [] ar1 = {4,4,4,4,4,4,0,4,4,4,4,4,4,0}; 
		assertTrue("Verify: state Board 1", equalsArr(ar1, board.getState()));	
		
		assertTrue("Verify: anChoose(2)", game.canChoose(2));
		assertTrue("Verify: choose(2)", board.choose(pl1.getSeat(),2));
		 /*    4   4   4   4  4  4 
		 * 0-  |   |   |   |  |  | -1
		 *     4   4   0   5  5  5
		    1*/
		int [] ar2 = {4,4,0,5,5,5,1,4,4,4,4,4,4,0}; 
		assertTrue("Verify: state Board 2", equalsArr(ar2, board.getState()));	
		
		assertTrue("Verify: canChoose(3)", game.canChoose(3));
		assertFalse("Verify: choose(3)", board.choose(pl1.getSeat(),3));
		 /*    4   4   4   4  5  5 
		 * 0-  |   |   |   |  |  | -2
		 *     4   4   0   0  6  6
		    2*/		
		int [] ar3 = {4,4,0,0,6,6,2,5,5,4,4,4,4,0}; 
		assertTrue("Verify: state Board 3", equalsArr(ar3, board.getState()));	
		
		assertTrue("Verify: anChoose(9)", game.canChoose(9));
		assertTrue("Verify: choose(9)", board.choose(pl2.getSeat(),9));
		 /*    5   5   5   0  5  5 
		 * 1-  |   |   |   |  |  | -2
		 *     4   4   0   0  6  6
		    2*/
		int [] ar4 = {4,4,0,0,6,6,2,5,5,0,5,5,5,1}; 
		assertTrue("Verify: state Board 4", equalsArr(ar4, board.getState()));	
		
		assertTrue("Verify: anChoose(8)", game.canChoose(8));
		assertTrue("Verify: choose(8)", board.choose(pl2.getSeat(),8));
		 /*    6   6   6   1  0  5 
		 * 2-  |   |   |   |  |  | -2
		 *     4   4   0   0  6  6
		    2*/
		int [] ar5 = {4,4,0,0,6,6,2,5,0,1,6,6,6,2}; 
		assertTrue("Verify: state Board 5", equalsArr(ar5, board.getState()));
		
		assertTrue("Verify: anChoose(7)", game.canChoose(7));
		assertFalse("Verify: choose(7)", board.choose(pl2.getSeat(),7));
		 /*    7   7   7   2  1  0 
		 * 2-  |   |   |   |  |  | -2
		 *     4   4   0   0  6  6
		    1*/
		int [] ar6 = {4,4,0,0,6,6,2,0,1,2,7,7,7,2}; 
		assertTrue("Verify: state Board 6", equalsArr(ar6, board.getState()));
		
		assertTrue("Verify: anChoose(5)", game.canChoose(5));
		assertFalse("Verify: choose(5)", board.choose(pl1.getSeat(),5));
		 /*    7   8   8   3  2  1 
		 * 2-  |   |   |   |  |  | -3
		 *     4   4   0   0  6  0
		    2*/
		int [] ar7 = {4,4,0,0,6,0,3,1,2,3,8,8,7,2}; 
		assertTrue("Verify: state Board 7", equalsArr(ar7, board.getState()));
		
		assertTrue("Verify: anChoose(9)", game.canChoose(9));
		assertFalse("Verify: choose(9)", board.choose(pl2.getSeat(),9));
		 /*    8   9   9   0  2  1 
		 * 2-  |   |   |   |  |  | -3
		 *     4   4   0   0  6  0
		    1*/
		int [] ar8 = {4,4,0,0,6,0,3,1,2,0,9,9,8,2}; 
		assertTrue("Verify: state Board 8", equalsArr(ar8, board.getState()));
		
		assertTrue("Verify: anChoose(1)", game.canChoose(1));
		assertFalse("Verify: choose(1)", board.choose(pl1.getSeat(),1));
		 /*    8   9   9   0  2  1 
		 * 2-  |   |   |   |  |  | -3
		 *     4   0   1   1  7  1
		 *     
		 *     8   9   9   0  2  1 
		 * 2-  |   |   |   |  |  | -4
		 *     4   0   1   1  7  0
		 *     
		 *     8   9   9   0  2  0 
		 * 2-  |   |   |   |  |  | -5
		 *     4   0   1   1  7  0
		    2*/
		int [] ar9 = {4,0,1,1,7,0,5,0,2,0,9,9,8,2}; 
		assertTrue("Verify: state Board 9", equalsArr(ar9, board.getState()));
		
		assertTrue("Verify: anChoose(8)", game.canChoose(8));
		assertFalse("Verify: choose(8)", board.choose(pl2.getSeat(),8));
		/*     8   9  10   1  0  0 
		 * 2-  |   |   |   |  |  | -5
		 *     4   0   1   1  7  0
		    1*/
		int [] ar10 = {4,0,1,1,7,0,5,0,0,1,10,9,8,2}; 
		assertTrue("Verify: state Board 10", equalsArr(ar10, board.getState()));
		
		assertTrue("Verify: anChoose(4)", game.canChoose(4));
		assertFalse("Verify: choose(4)", board.choose(pl1.getSeat(),4));
		/*     8  10  11   2  1  1 
		 * 2-  |   |   |   |  |  | -6
		 *     4   0   1   1  0  1
		    2*/
		int [] ar11 = {4,0,1,1,0,1,6,1,1,2,11,10,8,2}; 
		assertTrue("Verify: state Board 11", equalsArr(ar11, board.getState()));
		
		assertTrue("Verify: anChoose(9)", game.canChoose(9));
		assertFalse("Verify: choose(9)", board.choose(pl2.getSeat(),9));
		/*     8  11  12   0  1  1 
		 * 2-  |   |   |   |  |  | -6
		 *     4   0   1   1  0  1
		    1*/
		int [] ar12 = {4,0,1,1,0,1,6,1,1,0,12,11,8,2}; 
		assertTrue("Verify: state Board 12", equalsArr(ar12, board.getState()));
		
		assertTrue("Verify: anChoose(2)", game.canChoose(2));
		assertFalse("Verify: choose(2)", board.choose(pl1.getSeat(),2));
		/*     8  11  12   0  1  1 
		 * 2-  |   |   |   |  |  | -6
		 *     4   0   0   2  0  1
		    2*/
		int [] ar13 = {4,0,0,2,0,1,6,1,1,0,12,11,8,2}; 
		assertTrue("Verify: state Board 13", equalsArr(ar13, board.getState()));
		
		assertTrue("Verify: anChoose(7)", game.canChoose(7));
		assertFalse("Verify: choose(7)", board.choose(pl2.getSeat(),7));
		/*     8  11  12   0  2  0 
		 * 2-  |   |   |   |  |  | -6
		 *     4   0   0   2  0  1
		    1*/
		int [] ar14 = {4,0,0,2,0,1,6,0,2,0,12,11,8,2}; 
		assertTrue("Verify: state Board 14", equalsArr(ar14, board.getState()));
		
		assertTrue("Verify: anChoose(5)", game.canChoose(5));
		assertTrue("Verify: choose(5)", board.choose(pl1.getSeat(),5));
		/*     8  11  12   0  2  0 
		 * 2-  |   |   |   |  |  | -7
		 *     4   0   0   2  0  0
		    1*/
		int [] ar15 = {4,0,0,2,0,0,7,0,2,0,12,11,8,2}; 
		assertTrue("Verify: state Board 15", equalsArr(ar15, board.getState()));
		
		assertTrue("Verify: anChoose(3)", game.canChoose(3));
		assertFalse("Verify: choose(3)", board.choose(pl1.getSeat(),3));
		/*     8  11  12   0  2  0 
		 * 2-  |   |   |   |  |  | -8
		 *     4   0   0   0  1  0
		    2*/
		int [] ar16 = {4,0,0,0,1,0,8,0,2,0,12,11,8,2}; 
		assertTrue("Verify: state Board 16", equalsArr(ar16, board.getState()));
		
		assertTrue("Verify: anChoose(8)", game.canChoose(8));
		assertFalse("Verify: choose(8)", board.choose(pl2.getSeat(),8));
		/*     8  11  13   1  0  0 
		 * 2-  |   |   |   |  |  | -8
		 *     4   0   0   0  1  0
		    1*/
		int [] ar17 = {4,0,0,0,1,0,8,0,0,1,13,11,8,2}; 
		assertTrue("Verify: state Board 17", equalsArr(ar17, board.getState()));
		
		assertTrue("Verify: anChoose(0)", game.canChoose(0));
		assertFalse("Verify: choose(0)", board.choose(pl1.getSeat(),0));
		/*     8  11  13   1  0  0 
		 * 2-  |   |   |   |  |  | -8
		 *     0   1   1   1  2  0
		int [] ar18 = {0,1,1,1,2,0,8,0,0,1,13,11,8,2}; 
		assertTrue("Verify: state Board 18", equalsArr(ar18, board.getState()));
		
		assertTrue("Verify: anChoose(9)", game.canChoose(9));
		assertFalse("Verify: choose(9)", board.choose(9));
		/*     8  11  14   0  0  0 
		 * 2-  |   |   |   |  |  | -8
		 *     0   1   1   1  2  0
		    1*/
		int [] ar19 = {0,1,1,1,2,0,8,0,0,0,14,11,8,2}; 
		assertTrue("Verify: state Board 19", equalsArr(ar19, board.getState()));
		
		assertTrue("Verify: anChoose(4)", game.canChoose(4));
		assertTrue("Verify: choose(4)", board.choose(pl1.getSeat(),4));
		/*     8  11  14   0  0  0 
		 * 2-  |   |   |   |  |  | -9
		 *     0   1   1   1  0  1
		    1*/
		int [] ar20 = {0,1,1,1,0,1,9,0,0,0,14,11,8,2}; 
		assertTrue("Verify: state Board 20", equalsArr(ar20, board.getState()));
		
		assertTrue("Verify: anChoose(5)", game.canChoose(5));
		assertTrue("Verify: choose(5)", board.choose(pl1.getSeat(),5));
		/*     8  11  14   0  0  0 
		 * 2-  |   |   |   |  |  | -10
		 *     0   1   1   1  0  0
		    1*/
		int [] ar21 = {0,1,1,1,0,0,10,0,0,0,14,11,8,2}; 
		assertTrue("Verify: state Board 21", equalsArr(ar21, board.getState()));
		
		assertTrue("Verify: anChoose(1)", game.canChoose(1));
		assertFalse("Verify: choose(1)", board.choose(pl1.getSeat(),1));
		/*     8  11  14   0  0  0 
		 * 2-  |   |   |   |  |  | -10
		 *     0   0   2   1  0  0
		     2*/
		int [] ar22 = {0,0,2,1,0,0,10,0,0,0,14,11,8,2}; 
		assertTrue("Verify: state Board 22", equalsArr(ar22, board.getState()));
		
		assertTrue("Verify: anChoose(11)", game.canChoose(11));
		assertFalse("Verify: choose(11)", board.choose(pl2.getSeat(),11));
		/*     9  0  14   0  1  1 
		 * 6-  |   |   |   |  |  | -10
		 *     1   1   3   0  1  1
		     1*/
		int [] ar23 = {1,1,3,0,1,1,10,1,1,0,14,0,9,6}; 
		assertTrue("Verify: state Board 23", equalsArr(ar23, board.getState()));
		
		assertTrue("Verify: anChoose(1)", game.canChoose(1));
		assertFalse("Verify: choose(1)", board.choose(pl1.getSeat(),1));
		/*     9  0  14   0  1  1 
		 * 6-  |   |   |   |  |  | -10
		 *     1   0   4   0  1  1
		     2*/
		int [] ar24 = {1,0,4,0,1,1,10,1,1,0,14,0,9,6}; 
		assertTrue("Verify: state Board 24", equalsArr(ar24, board.getState()));
		
		assertTrue("Verify: anChoose(7)", game.canChoose(7));
		assertFalse("Verify: choose(7)", board.choose(pl2.getSeat(),7));
		/*     9  0  14   0  2  0 
		 * 6-  |   |   |   |  |  | -10
		 *     1   0   4   0  1  1
		     1*/
		int [] ar25 = {1,0,4,0,1,1,10,0,2,0,14,0,9,6}; 
		assertTrue("Verify: state Board 25", equalsArr(ar25, board.getState()));
		
		assertTrue("Verify: anChoose(5)", game.canChoose(5));
		assertTrue("Verify: choose(5)", board.choose(pl1.getSeat(),5));
		/*     9  0  14   0  2  0 
		 * 6-  |   |   |   |  |  | -11
		 *     1   0   4   0  1  0
		     1*/
		int [] ar26 = {1,0,4,0,1,0,11,0,2,0,14,0,9,6}; 
		assertTrue("Verify: state Board 26", equalsArr(ar26, board.getState()));
		
		assertTrue("Verify: anChoose(2)", game.canChoose(2));
		assertTrue("Verify: choose(2)", board.choose(pl1.getSeat(),2));
		/*     9  0  14   0  2  0 
		 * 6-  |   |   |   |  |  | -12
		 *     1   0   0   1  2  1
		     1*/
		int [] ar27 = {1,0,0,1,2,1,12,0,2,0,14,0,9,6}; 
		assertTrue("Verify: state Board 27", equalsArr(ar27, board.getState()));
		
		assertTrue("Verify: anChoose(5)", game.canChoose(5));
		assertTrue("Verify: choose(5)", board.choose(pl1.getSeat(),5));
		/*     9  0  14   0  2  0 
		 * 6-  |   |   |   |  |  | -13
		 *     1   0   0   1  2  0
		     1*/
		int [] ar28 = {1,0,0,1,2,0,13,0,2,0,14,0,9,6}; 
		assertTrue("Verify: state Board 28", equalsArr(ar28, board.getState()));
		
		assertTrue("Verify: anChoose(4)", game.canChoose(4));
		assertTrue("Verify: choose(4)", board.choose(pl1.getSeat(),4));
		/*     9  0  14   0  2  0 
		 * 6-  |   |   |   |  |  | -14
		 *     1   0   0   1  0  1
		     1*/
		int [] ar29 = {1,0,0,1,0,1,14,0,2,0,14,0,9,6}; 
		assertTrue("Verify: state Board 29", equalsArr(ar29, board.getState()));
		
		assertTrue("Verify: anChoose(5)", game.canChoose(5));
		assertTrue("Verify: choose(5)", board.choose(pl1.getSeat(),5));
		/*     9  0  14   0  2  0 
		 * 6-  |   |   |   |  |  | -15
		 *     1   0   0   1  0  0
		     1*/
		int [] ar30 = {1,0,0,1,0,0,15,0,2,0,14,0,9,6}; 
		assertTrue("Verify: state Board 30", equalsArr(ar30, board.getState()));
		
		assertTrue("Verify: anChoose(3)", game.canChoose(3));
		assertFalse("Verify: choose(3)", board.choose(pl1.getSeat(),3));
		/*     9  0  14   0  0  0 
		 * 6-  |   |   |   |  |  | -18
		 *     1   0   0   0  0  0
		     2*/
		int [] ar31 = {1,0,0,0,0,0,18,0,0,0,14,0,9,6}; 
		assertTrue("Verify: state Board 31", equalsArr(ar31, board.getState()));
		
		assertTrue("Verify: anChoose(10)", game.canChoose(10));
		assertFalse("Verify: choose(10)", board.choose(pl2.getSeat(),10));
		/*     10  2  1   1  1  1 
		 * 7-  |   |   |   |  |  | -18
		 *     2   1   1   1  1  1
		     1*/
		int [] ar32 = {2,1,1,1,1,1,18,1,1,1,1,2,10,7}; 
		assertTrue("Verify: state Board 32", equalsArr(ar32, board.getState()));
		
		assertTrue("Verify: anChoose(5)", game.canChoose(5));
		assertTrue("Verify: choose(5)", board.choose(pl1.getSeat(),5));
		/*     10  2  1   1  1  1 
		 * 7-  |   |   |   |  |  | -19
		 *     2   1   1   1  1  0
		      1*/
		int [] ar33 = {2,1,1,1,1,0,19,1,1,1,1,2,10,7}; 
		assertTrue("Verify: state Board 33", equalsArr(ar33, board.getState()));
		
		assertTrue("Verify: anChoose(4)", game.canChoose(4));
		assertFalse("Verify: choose(4)", board.choose(pl1.getSeat(),4));
		/*     10  2   1   1  1  0 
		 * 7-  |   |   |   |  |  | -21
		 *     2   1   1   1  0  0
		      2*/
		int [] ar34 = {2,1,1,1,0,0,21,0,1,1,1,2,10,7}; 
		assertTrue("Verify: state Board 34", equalsArr(ar34, board.getState()));
		
		assertTrue("Verify: anChoose(10)", game.canChoose(10));
		assertFalse("Verify: choose(10)", board.choose(pl2.getSeat(),10));
		/*     10  3   0   1  1  0 
		 * 7-  |   |   |   |  |  | -21
		 *     2   1   1   1  0  0
		      1*/
		int [] ar35 = {2,1,1,1,0,0,21,0,1,1,0,3,10,7}; 
		assertTrue("Verify: state Board 35", equalsArr(ar35, board.getState()));
		
		assertTrue("Verify: anChoose(3)", game.canChoose(3));
		assertFalse("Verify: choose(3)", board.choose(pl1.getSeat(),3));
		/*     10  3   0   1  0  0 
		 * 7-  |   |   |   |  |  | -23
		 *     2   1   1   0  0  0
		      2*/
		int [] ar36 = {2,1,1,0,0,0,23,0,0,1,0,3,10,7}; 
		assertTrue("Verify: state Board 36", equalsArr(ar36, board.getState()));
		
		assertTrue("Verify: anChoose(11)", game.canChoose(11));
		assertFalse("Verify: choose(11)", board.choose(pl2.getSeat(),11));
		/*     11  0   0   1  0  0 
		 * 8-  |   |   |   |  |  | -23
		 *     3   1   1   0  0  0
		      1*/
		int [] ar37 = {3,1,1,0,0,0,23,0,0,1,0,0,11,8}; 
		assertTrue("Verify: state Board 37", equalsArr(ar37, board.getState()));
		
		assertFalse("Verify: notWin", game.isWin());
		
		assertTrue("Verify: anChoose(0)", game.canChoose(0));
		assertFalse("Verify: choose(0)", board.choose(pl1.getSeat(),0));
		/*     11  0   0   0  0  0 
		 * 8-  |   |   |   |  |  | -25
		 *     0   2   2   0  0  0
		      2*/
		int [] ar38 = {0,2,2,0,0,0,25,0,0,0,0,0,11,8}; 
		assertTrue("Verify: state Board 38", equalsArr(ar38, board.getState()));
		
		assertTrue("Verify: isWin", game.isWin());
		assertTrue("Verify: getWiner()", game.getWiner() == EKindWins.WINDOWN);
		
		assertFalse("Verify: anNOTChoose(1) - no my Pot", game.canChoose(0));
		assertFalse("Verify: anNOTChoose(8) - countBeans in Pot = Zero", game.canChoose(8));
		assertFalse("Verify: anNOTChoose(-4) - Pot doesn't exist", game.canChoose(-4));
		assertFalse("Verify: anNOTChoose(24) - Pot doesn't exist", game.canChoose(24));
	}
	
	public void QtestShowTreeAI(){
		Logger.getInctanse().setLog(true);
		IBoard board = new Board(3, 2);	
		Game game = new Game(board);
		AIPlayer ai1 = new AIPlayer(EKindPlayer.COMP, game, 2, false);
		ai1.setLogTreeNodes(true);
		game.setPlayer1(ai1);
		AIPlayer ai2 = new AIPlayer(EKindPlayer.COMP, game, 2, false);
		game.setPlayer1(ai2);
		//game.start();
		ai1.getMove();
		if (!ai1.isReady()){}
		int pot = ai1.getChoose(); 
		System.out.println("pot=" + pot);
		boolean isAgainMove = board.choose(ai1.getSeat(), pot);
		Logger.getInctanse().setLog(false);
		if (isAgainMove){
			ai1.getMove();
			if (!ai1.isReady()){}
			pot = ai1.getChoose();
			System.out.println("pot=" + pot);
			isAgainMove = board.choose(ai1.getSeat(), pot);
		}else{
			ai2.getMove();
			if (!ai2.isReady()){}
			pot = ai2.getChoose();
			isAgainMove = board.choose(ai2.getSeat(), pot);
		}
		System.out.println(ai1.getCountTreeNodes());
	}
	
	
	public void testShowTreeAI2(){
		Logger.getInctanse().setLogTreeOfAI(true);
		Board board = new Board(3, 2);
		int [] ar = {2,3,1,4,6,3,1,3};
		board.setCountBeans(ar);
		Game game = new Game(board);
		AIPlayer ai1 = new AIAlphaBetaPlayer(EKindPlayer.COMP, game, 4, false);
		ai1.setLogTreeNodes(true);
		game.setPlayer1(ai1);
		AIPlayer ai2 = new AIPlayer(EKindPlayer.COMP, game, 2, false);
		game.setPlayer1(ai2);
		ai1.getMove();
		if (!ai1.isReady()){}
		int pot = ai1.getChoose(); 
		System.out.println("pot=" + pot);
		board.choose(ai1.getSeat(), pot);
		board.logBoard();
		System.out.println(ai1.getCountTreeNodes());
	}
}
