package org.amse.mARICa.game;

import java.util.ArrayList;

import org.amse.mARICa.Logger;
import org.amse.mARICa.model.*;
import org.amse.mARICa.model.impl.BantumiException;
import org.amse.mARICa.model.impl.players.AIPlayer;
import org.amse.mARICa.model.impl.players.NetPlayer;

public class Game extends Thread implements IGame {
	
	private IPlayer myPlayer1;
	private IPlayer myPlayer2;
	private IPlayer myCurrentPlayer;
	private IPlayer myOtherPlayer;
	private IBoard myBoard;
	private ArrayList<IGameListener> myListeners;
	private boolean myIsGameNow;
	
	public Game(IBoard board ){
		myBoard = board;	
		myListeners = new ArrayList<IGameListener>();
	}

	public boolean isWin() {
		//  
		int N = myBoard.getCountPots() * myBoard.getCountBeans();
		int man1 = myBoard.getCountInKalah1();
		int man2 = myBoard.getCountInKalah2();
		return ( (man1 > N) || 
				 (man2 > N) ||
				 (man1 == N && man2 == N)
				);
	}
	
	public EKindWins getWiner(){
		if (myBoard.getCountInKalah1() > myBoard.getCountInKalah2()){
			return EKindWins.WINDOWN;
		}
		if (myBoard.getCountInKalah1() < myBoard.getCountInKalah2()){
			return EKindWins.WINUP;
		}
		if (myBoard.getCountInKalah1() == myBoard.getCountInKalah2()){
			return EKindWins.NOWINNER;
		}
		return null;		
	}

	private void changePlayers(){
		if (myCurrentPlayer == myPlayer1){
			myCurrentPlayer = myPlayer2;
			myOtherPlayer = myPlayer1;
			myPlayer2.setCurrent(true);
			myPlayer1.setCurrent(false);
		}else{
			myCurrentPlayer = myPlayer1;
			myOtherPlayer = myPlayer2;
			myPlayer1.setCurrent(true);
			myPlayer2.setCurrent(false);
		}	
	}
	
	public IPlayer getCurrentPlayer() {
		return myCurrentPlayer;
	}

	private void printCurrentPlayer(){
		Logger.getInctanse().println("\n  " + myCurrentPlayer.getSeat());
	}
	
	public void run(){
		boolean isNetGame = 
					((myPlayer1.getKindPlayer() == EKindPlayer.HUMANLAN) ||
					(myPlayer2.getKindPlayer() == EKindPlayer.HUMANLAN) );
		//    - 
		myPlayer1.setCurrent(true);
		myCurrentPlayer = myPlayer1;
		myOtherPlayer = myPlayer2;
		//  
		callMessageListenersChangePlayer();
		myIsGameNow = true;// 
		int i = 1;
		try{
			do{
				Logger.getInctanse().println("Move is " + i++);
				//	 
				callMessageListenersStartTime();
				//,      
				myCurrentPlayer.getMove();
				if (!myIsGameNow){//  -   
					Logger.getInctanse().println("run myIsGameNow=" + myIsGameNow);
					callMessageListenersStopTime();
					myPlayer1.setStopGame(true);
					myPlayer2.setStopGame(true);				
					return;
				}
				printCurrentPlayer();
				//	    
				while(!(myCurrentPlayer.isReady())){
					if (!myIsGameNow){//  -   
						Logger.getInctanse().println("run myIsGameNow=" + myIsGameNow);
						callMessageListenersStopTime();
						myPlayer1.setStopGame(true);
						myPlayer2.setStopGame(true);
						return;
					}
					try {
						Thread.sleep(1000);
					} catch (InterruptedException e) {
						throw new BantumiException(e.getMessage()); 
					}
				}
				//	 
				callMessageListenersStopTime();
				int pot = myCurrentPlayer.getChoose();
				if (isNetGame){
					myOtherPlayer.setChoose(pot);
				}
				Logger.getInctanse().println(myCurrentPlayer.getSeat() + "  " + pot + " .");
				//	       
				//     
				boolean isNeedChangePlayer = !myBoard.choose(myCurrentPlayer.getSeat(), pot);
				myBoard.logBoard();
				callMessageListenersMoveDone();
				if (myCurrentPlayer.getKindPlayer() == EKindPlayer.COMP){
						Logger.getInctanse().StepPrint(myCurrentPlayer.getClass().getSimpleName());
						Logger.getInctanse().StepPrintln(" countNodes=" + ((AIPlayer)myCurrentPlayer).getCountTreeNodes());
				}
				if(isNeedChangePlayer){
					changePlayers();
					callMessageListenersChangePlayer();
				}            
			}while (!isWin());
		}catch (BantumiException ee){
			callMessageListenersHappenException(ee);
		}
		callMessageListenersWin();
	}

	private void callMessageListenersChangePlayer(){
		for(IGameListener i : myListeners){
			i.changePlayer(myCurrentPlayer.getSeat());
		}
	}

	private void callMessageListenersMoveDone() throws BantumiException{
		for(IGameListener i : myListeners){
			i.moveDone(myBoard.getMoves());
		}
	}
	
	private void callMessageListenersWin(){
		for(IGameListener i : myListeners){
			i.changePlayer(null);
		}
		for(IGameListener i : myListeners){
			i.win();
		}		
	}
	
	private void callMessageListenersStartTime(){
		for(IGameListener i : myListeners){
			i.startTime();
		}
	}
	
	public void callMessageListenersStopTime(){
		for(IGameListener i : myListeners){
			i.stopTime();
		}
	}
	
	private void callMessageListenersHappenException(BantumiException ee){
		for(IGameListener i : myListeners){
			i.happenException(ee);
		}
	}
	
	public boolean canChoose(int k) {
		return myBoard.canChoose(myCurrentPlayer.getSeat(), k);
	}

	public ESeatPlayer getSeatPlayer(IPlayer player) {
		if (player == myPlayer1){
			return ESeatPlayer.DOWN;
		}
		return ESeatPlayer.UP;
	}

	public IBoard getBoard() {
		return myBoard;
	}

	public IPlayer getPlayer1() {
		return myPlayer1;
	}
	
	public IPlayer getPlayer2() {
		return myPlayer2;
	}

	public void setPlayer1(IPlayer player1) {
		myPlayer1 = player1;
		myPlayer1.setSeat(ESeatPlayer.DOWN);		
	}

	public void setPlayer2(IPlayer player2) {
		myPlayer2 = player2;
		myPlayer2.setSeat(ESeatPlayer.UP);
	}
	
	public void addListener(IGameListener o) {
		myListeners.add(o);		
	}

	public boolean removeListener(IGameListener o) {
		return myListeners.remove(o);		
	}
	
	public void setIsGameNow(boolean isGameNow){
		myIsGameNow = isGameNow;
		Logger.getInctanse().println("setIsGameNow " + isGameNow);
		if (!isGameNow && (myCurrentPlayer.getKindPlayer() == EKindPlayer.HUMANLAN)){
			((NetPlayer)myCurrentPlayer).setInitiativeStopGame(true);
			try {
				myCurrentPlayer.setStopGame(true);
			} catch (BantumiException e) {
				callMessageListenersHappenException(e);
			}			
		}		
	}

	public void reverseSeatPlayer() {
		myPlayer1.reverseSeat();
		myPlayer2.reverseSeat();
		IPlayer pl = myPlayer1;
		myPlayer1 = myPlayer2;
		myPlayer2 = pl;		
	}
}
