package org.amse.mARICa.model.impl.players;

import java.util.*;

import org.amse.mARICa.Logger;
import org.amse.mARICa.game.*;
import org.amse.mARICa.model.*;

public class AIPlayer extends AbstractPlayer{

	protected ESeatPlayer myHisSeat;
	protected IBoard myCopyBoard;
	protected int myMaxLevel;
	protected boolean myIsRandomChoose;
	protected int myLevelZeroAgain;
	protected int myCountTreeNodes = 0;
	/**
	 *  [0]    
	 *    -    ,
	 *    .   -   ,
	 *      -    
	 *    level = 0
	 */
	protected ArrayList<Integer> myBranchChoose;
	protected boolean myIsLogTreeNodes;
	
	public AIPlayer(EKindPlayer kind, IGame game) {
		super(kind, game);
		myMaxLevel = 2;
		myIsRandomChoose = false;		
	}
	
	public AIPlayer(EKindPlayer kind, IGame game, int maxLevel, boolean rand) {
		super(kind, game);
		myIsRandomChoose = rand;	
		myMaxLevel = maxLevel;
	}
	
	public void setSeat(ESeatPlayer seat){
		mySeat = seat;
		if (mySeat == ESeatPlayer.UP)
			myHisSeat = ESeatPlayer.DOWN;
		else
			myHisSeat = ESeatPlayer.UP;
	}
	
	public int getMaxLevel(){
		return myMaxLevel;
	}
	
	public void setMaxLevel(int level){
		myMaxLevel = level;
	}

	public void getMove() {
		myIsCurrent = true;
		myIsReady = false;
		myBranchChoose = new ArrayList<Integer>();
		findOptimumPot();
		myIsReady = true;
		if (!myIsRandomChoose){
			//    
			myChoosingPot = myBranchChoose.get(1);
			return;
		}
		Random rand = new Random();
		int index = rand.nextInt(myBranchChoose.size()-1)+1;
		myChoosingPot = myBranchChoose.get(index);
		myIsReady = true;
	}
	
	protected  void findOptimumPot(){
		myCopyBoard = (IBoard)myGame.getBoard().clone();
		myLevelZeroAgain = 0;
		myCountTreeNodes = 0;
		findInRecursy(-1, myCopyBoard, 0);
	}
	
	protected int findInRecursy(int pot, IBoard board, int level){
		Logger log = Logger.getInctanse();
		if (myStopGame){
			return -100;
		}
		if (pot >= 0){
			boolean isAgainMove = choose(board, level, pot);
			myCountTreeNodes++;			
			if (myIsLogTreeNodes){
				if (level % 2 == 0){
					log.AIprintln("\n" + myHisSeat + " player choose " + pot + " pot on level=" +  (level - 1 ) + "  myCountTreeNodes=" + myCountTreeNodes);
				}else{
					log.AIprintln("\n" + mySeat + " player choose " + pot + " pot on level=" + (level - 1 ) + "  myCountTreeNodes=" + myCountTreeNodes);
				}
				board.logBoard();				
			}
			if (isAgainMove){
				if (level == 1){
					//           myBranchChoose
					myLevelZeroAgain++;
					int eval = findInRecursy(-1, board, level-1);
					myLevelZeroAgain--;
					return eval;
				}
				return findInRecursy(-1, board, level-1);
			}
		}
		int s;
		int eval = 0;
		if (level == myMaxLevel || board.isFinishGame()) {
			s = calcStatCritFunc(board);
			log.AIprintln("Stat = " + s + "  FinishGame=" + board.isFinishGame());
			return s;
		}else{
			eval = initDynCritFunc(level);			
			for(int ipot = 0; ipot < myCopyBoard.getCountPots(); ipot++){
				if (myStopGame){
					return -100;
				}
				if (canChoose(board, level, ipot)){
					s = findInRecursy(ipot, (IBoard)board.clone(), level+1);
					log.AIprint("eval=" + eval + " s=" + s);
					eval = calcDynCritFunc(level, s, eval, ipot);
					log.AIprintln(" => eval=" + eval);
				}
			}		
		}
		return eval;
	}

	protected int initDynCritFunc(int level) {
		int eval;
		if(level == 0 && myLevelZeroAgain == 0){
			myBranchChoose.add(0, Integer.MIN_VALUE);
		}
		if(level % 2 == 0){
			eval = Integer.MIN_VALUE;
		}else{
			eval = Integer.MAX_VALUE;
		}
		return eval;
	}

	protected int calcDynCritFunc(int level, int s, int eval, int ipot) {
		if (level % 2 != 0){
			eval = Math.min(s, eval);
		}else{
			eval  = Math.max(s, eval);
			if(level == 0 && myLevelZeroAgain == 0){
				if (s == myBranchChoose.get(0)){
					myBranchChoose.add(ipot);
				}else if(myBranchChoose.get(0) < s){
					myBranchChoose.clear();
					myBranchChoose.add(0, eval);
					myBranchChoose.add(ipot);
					eval = myBranchChoose.get(0);
				}									
			}
		}
		return eval;
	}
	/**
	 *  :      
	 */
	protected int calcStatCritFunc(IBoard board){
		int ret = board.getCountInKalah1() - board.getCountInKalah2();
		if (mySeat == ESeatPlayer.DOWN){
			return ret;
		}else{
			return -ret;
		}
	}
	/**
	 *  -     
	 *       
	 *   -  this AI
	 *   -  
	 */
	protected boolean canChoose(IBoard board, int level, int ipot){
		if (level % 2 == 0){
			return board.canChoose(mySeat, ipot);
		}else{
			return board.canChoose(myHisSeat, ipot);
		}
	}
	/**
	 *    .
	 *      ,
	 *    :
     *   -  this AI,
     *   -  
     * (   ,  level  )
     * return   
	 */
	protected boolean choose(IBoard board, int level, int pot){
		if (level % 2 == 0){
			return board.choose(myHisSeat, pot);
		}else{
			return board.choose(mySeat, pot);
		}
	}

	public boolean isRandomChoose() {
		return myIsRandomChoose;
	}

	public void setRandomChoose(boolean myIsRandomChoose) {
		this.myIsRandomChoose = myIsRandomChoose;
	}

	public void setStopGame(boolean stopGame) {
		myStopGame = stopGame;		
	}

	public void setLogTreeNodes(boolean b){
		myIsLogTreeNodes = b;
	}
	
	public int getCountTreeNodes(){
		return myCountTreeNodes;
	}
}
