package org.amse.mARICa.model.impl.players;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.SocketTimeoutException;
import java.net.UnknownHostException;

import org.amse.mARICa.Logger;
import org.amse.mARICa.game.IGame;
import org.amse.mARICa.model.EBantumiException;
import org.amse.mARICa.model.EKindPlayer;
import org.amse.mARICa.model.impl.BantumiException;
import org.amse.mARICa.model.impl.Settings;
import org.amse.mARICa.view.ButtonPanel;

public class NetPlayer extends AbstractPlayer{

	private boolean myIsServer;
	private String myIP;
	private int myPort;
	private int myReceiveCountBeans;
	private ServerSocket myServerSocket;
	private Socket myClientSocket;
	private PrintWriter myOut;
	private BufferedReader myIn;
	private ButtonPanel myListenerStopGame;
	private int timeout = 2000; //2 
	private boolean isMyInitiativeStopGame = false;

	public NetPlayer(EKindPlayer kind, IGame game, boolean isServer, String ip, int port) {
		super(kind, game);
		myIsServer = isServer;
		myIP = ip;
		myPort = port;
	}
	
	public void setListenerStopGame(ButtonPanel bp){
		myListenerStopGame = bp;
	}
	/**
	 *  ServerSocket
	 * @param port  
	 * @throws BantumiException
	 */          
	private void init(int port) throws BantumiException{
		myServerSocket = null;
        try {
        	myServerSocket = new ServerSocket(port);        	
        } catch (IOException e) {
        	throw new BantumiException(EBantumiException.PORT_BUSY, e.getMessage());
        }
        myClientSocket = null;
        try {
            myClientSocket = myServerSocket.accept();
            myClientSocket.setSoTimeout(timeout);
        } catch (IOException e) {
        	throw new BantumiException(e.getMessage());
        }
        try {
			myOut = new PrintWriter(myClientSocket.getOutputStream(), true);
			myIn = new BufferedReader(new InputStreamReader(myClientSocket.getInputStream()));
		} catch (IOException e) {
			throw new BantumiException(e.getMessage()); 
		}
		checkSetsByServer();
	}
	
	public void init() throws BantumiException {
		if (myIsServer){
			init(myPort);
		}else{
			init(myIP, myPort);
		}		
	}
	
	/**
	 *  Client'
	 * @param ip 
	 * @param port     
	 * @throws BantumiException 
	 * @throws Exception 
	 */
	private void init(String ip, int port) throws BantumiException {
		try {
			myClientSocket = new Socket(ip, port);
			myClientSocket.setSoTimeout(timeout);
            myOut = new PrintWriter(myClientSocket.getOutputStream(), true);
            myIn = new BufferedReader(new InputStreamReader(myClientSocket.getInputStream()));
        } catch (UnknownHostException e) {
            throw new BantumiException(EBantumiException.IP_WRONG, e.getMessage());        	
        } catch (IOException e) {
        	throw new BantumiException(e.getMessage());  
        }
        checkSetsByClient();
	}
	
	private void checkSetsByServer() throws BantumiException{
		sendCountBeans();
		sendMySeat();
		if (!receiveAnswer()){
			throw new BantumiException(EBantumiException.ERROR_SENT); 			
		}
	}
	
	private void checkSetsByClient() throws BantumiException{
		 boolean count = receiveCountBeans();
	     boolean seat = receiveMySeat();
	     if (!count && !seat){
	       	throw new BantumiException(EBantumiException.COUNT_BEANS_SEAT_WRONG);
	     }
	     if (!count){
	      	throw new BantumiException(EBantumiException.COUNT_BEANS_WRONG);
	     }
	     if (!seat){
	      	throw new BantumiException(EBantumiException.SEAT_WRONG);
	     }
	     sendAnswerYes();
	}
	
	/**
	 *   ,     
	 *     .    
	 *  Exception (    UI  )
	 *        UI  sendAnswer()
	 */
	public void sendAnswerYes(){
		if (myOut != null){
			myOut.println("yes");
			Logger.getInctanse().println("Sent: yes");
		}
	}
	
	/**
	 *         
	 *   () ,     
	 *   
	 */
	public void sendAnswerNo(){
		Logger.getInctanse().println("Sent: no");
		myOut.println("no");
	}
	
	private boolean receiveAnswer() throws BantumiException{
		String ans = "";
		try {
			do{
				try{
					ans = myIn.readLine();
				}catch (SocketTimeoutException ste){
					Logger.getInctanse().println(ste.getMessage());
				}				
				if (myStopGame){
					 return false;
				}
			}while (ans == "");
		} catch (IOException e) {
			throw new BantumiException(e.getMessage());  
		}
		if (ans.equals("no")){			
			Logger.getInctanse().println("receive no");
			throw new BantumiException(EBantumiException.NO_WANT_SETTINFS);
		}else if (!ans.equals("yes")){
			throw new BantumiException(EBantumiException.ERROR_SENT);
		}
		Logger.getInctanse().println("receive yes");
		return true;
	}
	
	private void sendCountBeans(){
		String sendCountBeans = "CountBeans " + Settings.getCountBeans();
		myOut.println(sendCountBeans);
		Logger.getInctanse().println("Send: " + sendCountBeans);
	}
	
	private boolean receiveCountBeans() throws BantumiException{
		String countBeans = "";
		try {
			do{
				try{
					countBeans = myIn.readLine();
				}catch (SocketTimeoutException ste){
					Logger.getInctanse().println(ste.getMessage());
				}				
				if (myStopGame){
					 return false;
				}
			}while (countBeans == "");
		} catch (IOException e) {
			throw new BantumiException(e.getMessage());  
		}
		if (!countBeans.startsWith("CountBeans ")){
			throw new BantumiException(EBantumiException.ERROR_SENT);
		}
		String count = countBeans.substring("CountBeans ".length());
		myReceiveCountBeans = Integer.parseInt(count);
		Logger.getInctanse().println("receive: " + countBeans);
		if (myReceiveCountBeans != Settings.getCountBeans()){
			return false;
		}		
		return true;
	}
	
	public int getReceiveCountBeans(){
		return myReceiveCountBeans;
	}
	
	private void sendMySeat(){
		String sendMySeat = "I'm " + mySeat;		
		myOut.println(sendMySeat);
		Logger.getInctanse().println("Send: " + sendMySeat);
	}
	
	private boolean receiveMySeat() throws BantumiException{
		String receiveSeat = "";
		try {
			do{
				try{
					receiveSeat = myIn.readLine();
				}catch (SocketTimeoutException ste){
					Logger.getInctanse().println(ste.getMessage());
				}				
				if (myStopGame){
					 return false;
				}
			}while (receiveSeat == "");
		} catch (IOException e) {
			throw new BantumiException(e.getMessage());  
		}
		if (!receiveSeat.startsWith("I'm ")){
			throw new BantumiException(EBantumiException.ERROR_SENT);
		}
		String seat = receiveSeat.substring("I'm ".length());
		if (seat.equals(mySeat.toString())){
			return false;
		}
		Logger.getInctanse().println("receive: " + receiveSeat);
		return true;      
	}
	
	public void getMove() throws BantumiException {
		myIsCurrent = true;
		String fromOtherPlayer = null;
		//	,    . 
		try {	
			do {
				try{
					fromOtherPlayer = myIn.readLine();
				}catch (SocketTimeoutException ste){
					Logger.getInctanse().println(ste.getMessage());
				}catch (IOException e) {//connection failed
					Logger.getInctanse().println(e.getMessage());
					setStopGame(true);
					throw new BantumiException(e.getMessage());  
					//throw new NetException(ENetException.CONNECTION_FAILED)					
				}
				 if (fromOtherPlayer == null){
					 myIsCurrent = true;
					 Logger.getInctanse().println("");
					 if (myStopGame){
						 Logger.getInctanse().println("NP.getMove() myStopGame=" + myStopGame + "=> RETURN");
						 return;
					 }
				 }
			 }while (fromOtherPlayer == null);
			 myChoosingPot = Integer.valueOf(fromOtherPlayer).intValue();
			 myIsReady = true;
		} catch (NumberFormatException e) {
			if (fromOtherPlayer.equals("stop")){
				Logger.getInctanse().println("NetPlayer receive STOP");
				myStopGame = true;
				myListenerStopGame.cancelGameOtherPlayer();
				return;
			}else{
				throw new BantumiException(EBantumiException.ERROR_SENT);
			}
		}		
	}
	
	public void setChoose(int pot){
		myIsReady = true;
		if (!myIsCurrent){
			myChoosingPot = pot;			
			//   .
			myOut.println(pot);
		}		
	}
	
	public void setInitiativeStopGame(boolean b){
		isMyInitiativeStopGame = b;
	}
	
	public void setStopGame(boolean stopGame) throws BantumiException{
		Logger.getInctanse().println("NetPlayer myStopGame=" + myStopGame + " setStopGame=" + stopGame);
		if (!myStopGame && stopGame && (myOut != null) ){
			Logger.getInctanse().println("NetPlayer sent STOP");
			myOut.println("stop");
		}
		if (!isMyInitiativeStopGame){
			closeStream();
		}		
		myStopGame = stopGame;
	}
	
	public IGame getGame() {
		return myGame;
	}
	
	private void closeStream() throws BantumiException{
		Logger.getInctanse().println("close Stream");
		if ( myOut != null){
			myOut.close();
		}
		try {
			if (myIn != null){
				myIn.close();
			}
			if (myClientSocket != null){
				myClientSocket.close();
			}
			if (myIsServer && myServerSocket != null){
				myServerSocket.close();
			}
		} catch (IOException e) {
			throw new BantumiException(e.getMessage());  
		}		
	}
	
	public int getPort(){
		return myPort;
	}	
}
