package org.amse.mARICa.view;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.*;

import org.amse.mARICa.Main;
import org.amse.mARICa.game.AbstractGameListener;
import org.amse.mARICa.game.IGameListener;
import org.amse.mARICa.model.ESeatPlayer;
/**
 *   - ,    
 *   -   BevelBorder.RAISED   
 * @author mARICa
 *
 */
public class ArrowPanel extends JPanel {

	private static final long serialVersionUID = 1L;

	private JButton myUp;
	private JButton myDown;
	private JLabel myTimeLabel;
	private ImageIcon myIconUp;
	private ImageIcon myIconDown;
	private ImageIcon myIconNoAct;
	private final int TIMER_DELAY = 1000;
	
	public ArrowPanel (int iconPotHeight){
		super();
		
		this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		myUp = new JButton();		
		myDown = new JButton();
		myTimeLabel = new JLabel("- -:- -");
		
		ClassLoader cl = Main.class.getClassLoader();		
		myIconUp = new ImageIcon(cl.getResource("images/arrowup.png"));
		myIconDown = new ImageIcon(cl.getResource("images/arrowdown.png"));
		myIconNoAct = new ImageIcon(cl.getResource("images/arrownoact.png"));
		
		myUp.setIcon(myIconUp);
		myDown.setIcon(myIconDown);
		
		Dimension dim = new Dimension (myTimeLabel.getPreferredSize().width, myIconUp.getIconHeight() + 5);
		myUp.setPreferredSize(dim);
		myDown.setPreferredSize(dim);
		
		dim = new Dimension( dim.width + 7, iconPotHeight);
		this.setPreferredSize(dim);
		this.setMaximumSize(dim);
		
		myUp.setAlignmentX(JComponent.CENTER_ALIGNMENT);
		myDown.setAlignmentX(JComponent.CENTER_ALIGNMENT);
		myTimeLabel.setAlignmentX(JComponent.CENTER_ALIGNMENT);
		
		this.add(myUp);
		this.add(Box.createVerticalGlue());
		this.add(myTimeLabel);
		this.add(Box.createVerticalGlue());
		this.add(myDown);

	}
	
	public IGameListener createGameListener(){
		return new GameListener();
	}
	
	/**
	 *   
	 * /  
	 * @author mARICa
	 *
	 */
	private class GameListener extends AbstractGameListener implements ActionListener {
		
		private int myTime = 0;
		private final Timer myTimer;

		public GameListener() {
			this.myTimer = new Timer(TIMER_DELAY, this);
		}
		
		public void changePlayer(ESeatPlayer current){
			if (current == null){
				myUp.setIcon(myIconNoAct);
				myUp.setIcon(myIconUp);	
				return;
			}
			if (current == ESeatPlayer.DOWN){
				myUp.setIcon(myIconNoAct);
				myDown.setIcon(myIconDown);
			}else{
				myDown.setIcon(myIconNoAct);
				myUp.setIcon(myIconUp);			
			}
		}	
	

		public void stopTime() {
			this.myTimer.stop();
			myTimeLabel.setText("- -:- -");
			myTime = 0;
		}

		public void startTime() {
			myTime = 0;
			myTimeLabel.setText(this.timeToString());
			myTimer.start();
			ArrowPanel.this.repaint();
		}

		public void actionPerformed(ActionEvent e) {
			this.myTime++;
			myTimeLabel.setText(this.timeToString());
			ArrowPanel.this.repaint();
		}

		private String timeToString() {
			final int sec = this.myTime % 60;
			final int minute = ((this.myTime - sec) / 60) % 60;
			StringBuffer sb = new StringBuffer();
			if (minute < 10) {
				sb.append("0");
			}
			sb.append(minute);
			sb.append(":");
			if (sec < 10) {
				sb.append("0");
			}
			sb.append(sec);
			return sb.toString();
		}

	}
}
