package org.amse.mARICa.view;

import java.awt.Point;
import java.util.List;

import javax.swing.ImageIcon;
import javax.swing.JLabel;

import org.amse.mARICa.Logger;
import org.amse.mARICa.Main;
import org.amse.mARICa.game.AbstractGameListener;
import org.amse.mARICa.game.IGameListener;
import org.amse.mARICa.model.ESeatPlayer;
import org.amse.mARICa.model.IModelListener;
import org.amse.mARICa.model.impl.BantumiException;
import org.amse.mARICa.model.impl.Move;
import org.amse.mARICa.model.impl.Settings;

public class HandLabel extends JLabel {

	private static final long serialVersionUID = 1L;
	private ImageIcon myPalmDown;
	private ImageIcon myPalmUp;
	private ImageIcon myPalmLeft;
	private ImageIcon myPalmRight;
	private ImageIcon myFistDown;
	private ImageIcon myFistUp;
	private ImageIcon myFistLeft;
	private ImageIcon myFistRight;
	
	private final PotLabel [] myPotsLabel;
	private Point [] myLocationPots;// Location   Pot
	private int offset = 10;//
	private int myCountUpOreDown;// 
	private final int COEFFICIENT_MILIS = 20;
	private long millis = Settings.getHandMinDelay();
	private final int myKalah1;
	private final int myKalah2;
	private int myPreviousPot;
	private IModelListener myBoardListener;
	private Point myZero;//  -    
	
	{
		ClassLoader cl = Main.class.getClassLoader();
		myPalmDown = new ImageIcon(cl.getResource("images/hand/palmdown.png"));		
		myPalmUp = new ImageIcon(cl.getResource("images/hand/palmup.png"));
		myPalmLeft = new ImageIcon(cl.getResource("images/hand/palmleft.png"));
		myPalmRight = new ImageIcon(cl.getResource("images/hand/palmright.png"));
		
		myFistDown = new ImageIcon(cl.getResource("images/hand/fistdown.png"));
		myFistUp =  new ImageIcon(cl.getResource("images/hand/fistup.png"));
		myFistLeft =  new ImageIcon(cl.getResource("images/hand/fistleft.png"));
		myFistRight =  new ImageIcon(cl.getResource("images/hand/fistright.png"));		
	}
	
	public HandLabel(IModelListener boardListener, PotLabel [] potsLabel){
		super();
		myBoardListener = boardListener;
		myPotsLabel = potsLabel;
		myKalah1 = myPotsLabel.length / 2 - 1;
		myKalah2 = myPotsLabel.length - 1;
		int w = myPalmDown.getIconWidth();
		int h = myPalmDown.getIconHeight();
		int max = Math.max(w, h);
		setBounds(0, 0, max, max);		
	}
	
	public void setLocation(int x, int y){
		 super.setLocation(x+myZero.x, y+myZero.y);		 
	}
	 
	public void setLocation(Point p){
		 super.setLocation(p.x+myZero.x, p.y+myZero.y);		 
	}
		
	public Point getLocation(){
		 Point p = super.getLocation();
		 p.x -= myZero.x;
		 p.y -= myZero.y;
		 return p;
	}
	
	public void setSpeed(){
		 millis = Settings.getHandMinDelay() + COEFFICIENT_MILIS * (Settings.getSpeedHand() - 1);
		 Logger.getInctanse().println("HandLabel speed = " + millis);
	}
	
	public IGameListener createGameListener(){
		return new GameListener();
	}
	
	private void createLocationForEveryPot(){
		int mainYUp = 30;
		int mainYDown = 70;
		//..    
		int widthPot = myPotsLabel[0].getWidth();
		int widthHand = getWidth();
		myLocationPots = new Point [myPotsLabel.length];
		int x = 0,y = 0;
		// 
		y = mainYDown;
		for (int i = 0; i < myKalah1; i++){
			x = (widthPot + 5) + myPotsLabel[i].getLocation().x + ( (widthPot - widthHand) /2);
			myLocationPots[i] = new Point(x,y);
		}
		// 
		y = mainYUp;
		for (int i = myKalah1+1; i < myKalah2; i++){
			x = (widthPot + 5) + myPotsLabel[i].getLocation().x + ( (widthPot - widthHand) /2);
			myLocationPots[i] = new Point(x,y);
		}
		// 1
		myLocationPots[myKalah1] = new Point(260,55);
		// 2
		myLocationPots[myKalah2] = new Point(20,55);	
		//      ,  -   
		myCountUpOreDown =  myPotsLabel[0].getHeight() / 2 / offset;		
	}
	
	public void setStartPos(Point locBoard){
		myZero = locBoard;
		createLocationForEveryPot();
		setIcon(myPalmDown);
		setLocation(myLocationPots[0]);
		myPreviousPot = 0;
	}
	
	
	private class GameListener extends AbstractGameListener {

		public void moveDone(List<Move> moves) throws BantumiException {
			try{
				int i;
				for (i = 0; i < moves.size()-1; i++){
					Move move = moves.get(i);
					if (move.isSourse()){
						setPalmIcon(move.getPot());
					}
					goToPot(move.getPot());
					downAndUpToPot(move, false);
					myPreviousPot = moves.get(i).getPot();
				}
				goToPot(moves.get(i).getPot());
				downAndUpToPot(moves.get(i), true);
				myPreviousPot = moves.get(i).getPot();		
			}catch(InterruptedException e){
				throw new BantumiException(e.getMessage()); 
			}			
			//"  " +   ,    
			myBoardListener.statePotsChanged();	
			//         
			try {
				Thread.sleep(millis);
			} catch (InterruptedException e) {
				throw new BantumiException(e.getMessage()); 
			}
		}
		/**
		 *   - 
		 * @param pot     ,   
		 */
		private void setPalmIcon(int pot){
			if (pot < myKalah1){
				setIcon(myPalmDown);
			}else if (pot == myKalah1){
				setIcon(myPalmRight);
			}else if (pot == myKalah2){
				setIcon(myPalmLeft);
			}else{//myKalah1 < pot < myKalah2
				setIcon(myPalmUp);
			}
		}
		/**
		 *   - 
		 * @param pot     ,   
		 */
		private void setFistIcon(int pot){
			if (pot < myKalah1){
				setIcon(myFistDown);
			}else if (pot == myKalah1){
				setIcon(myFistRight);
			}else if (pot == myKalah2){
				setIcon(myFistLeft);
			}else{//myKalah1 < pot < myKalah2
				setIcon(myFistUp);
			}
		}
		
		private void goToKalah(int k, int targetX, ImageIcon iconIf, ImageIcon iconElse, int kakah ) throws InterruptedException{
			while(k*getLocation().x > targetX){
				setLocation( Math.abs(k*getLocation().x - offset), myLocationPots[myPreviousPot].y);
				Thread.sleep(millis);
			}
			if (getIcon() == myPalmUp || getIcon() == myPalmDown){
				setIcon(iconIf);
			}else{//myFistUp || myFistDown
				setIcon(iconElse);
			}
			setLocation(myLocationPots[kakah]);
			Thread.sleep(millis);
		}
		private void goFromKalah(int pot, ImageIcon iconIf, ImageIcon iconElse, int pot1, int pot2, int k ) throws InterruptedException{
			if (getIcon() == iconIf){
				setPalmIcon(pot);
			}else if (getIcon() == iconElse){
				setFistIcon(pot);
			}
			if (pot < myKalah1){
				setLocation(myLocationPots[pot1]);
			}else{//myKalah2 > pot > myKalah1  
				setLocation(myLocationPots[pot2]);
			}
			Thread.sleep(millis);
			while (k*getLocation().x > k*myLocationPots[pot].x ){//
				setLocation(Math.abs(k*getLocation().x - offset), myLocationPots[pot].y);
				Thread.sleep(millis);
			}
		}
		
		private void goToPot(int pot) throws InterruptedException{
			if (pot == myKalah2){
				goToKalah(1, myLocationPots[myKalah2-1].x, myPalmLeft, myFistLeft, myKalah2);				
			}else if (pot == myKalah1){	
				goToKalah(-1, -myLocationPots[myKalah1-1].x, myPalmRight, myFistRight, myKalah1);	
			}else if (myPreviousPot == myKalah2){
				goFromKalah(pot, myPalmLeft, myFistLeft, 0, myKalah2-1, -1);				
			}else if (myPreviousPot == myKalah1){
				goFromKalah(pot, myPalmRight, myFistRight, myKalah1-1, myKalah1+1, 1);				
			}else{// 
				while ( getLocation().x < myLocationPots[pot].x ){
					setLocation(getLocation().x + offset, myLocationPots[pot].y);
					Thread.sleep(millis);
				}
				while (getLocation().x > myLocationPots[pot].x ){
					setLocation(getLocation().x - offset, myLocationPots[pot].y);
					Thread.sleep(millis);
				}
				setLocation(myLocationPots[pot]);
			}
		}
		
		private void downAndUpToKalah(int pot, int count, boolean isLastPot, int offs, ImageIcon icon1, ImageIcon icon2) throws InterruptedException{
			setIcon(icon1);
			setLocation(myLocationPots[pot]);
			moveUpOreDown(offs, 0);
			setIcon(icon2);
			Thread.sleep(millis);
			myBoardListener.statePotChanged(pot, count);
			setIcon(icon1);
			if (isLastPot){
				setPalmIcon(pot);
			}
			moveUpOreDown(-offs, 0);
		}	
		
		private void downAndUpToPotSeat(int pot, int count, boolean isLastPot, int offs, boolean isSourse, ImageIcon icon1, ImageIcon icon2 ) throws InterruptedException{
			moveUpOreDown(0, offs);
			if (!isSourse){
				setIcon(icon1);//
			}
			Thread.sleep(millis);
			//   
			myBoardListener.statePotChanged(pot, count);
			setIcon(icon2);//
			Thread.sleep(millis);	
			if (isLastPot){
				setPalmIcon(pot);
			}
			moveUpOreDown(0, -offs);
		}		
		
		/**
		 *   ,      
		 * @param isLastPot  -  true,    
		 */
		private void downAndUpToPot(Move move, boolean isLastPot) throws InterruptedException{
			int pot = move.getPot();
			int count = move.getCountBeans();
			if (pot == myKalah1){
				downAndUpToKalah(myKalah1, count, isLastPot, offset, myFistRight, myPalmRight);
			}else if (pot == myKalah2){
				downAndUpToKalah(myKalah2, count, isLastPot, -offset, myFistLeft, myPalmLeft);
			}else if (myPotsLabel[pot].getSeat() == ESeatPlayer.DOWN){
				downAndUpToPotSeat(pot, count, isLastPot, offset, move.isSourse(), myPalmDown, myFistDown );
			}else{//== ESeatPlayer.UP
				downAndUpToPotSeat(pot, count, isLastPot, -offset, move.isSourse(), myPalmUp, myFistUp );
			}
		}
		/**
		 *   /    
		 * @param deltaX -    
		 * @param deltaY -    Y
		 * @throws InterruptedException
		 */
		private void moveUpOreDown(int deltaX, int deltaY) throws InterruptedException {
			for (int i = 0; i < myCountUpOreDown; i++){
				setLocation(getLocation().x + deltaX, getLocation().y + deltaY);
				Thread.sleep(millis);					
			}
		}				
	}

}
