package org.amse.mARICa.view;

import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.net.URL;

import javax.swing.*;
import javax.swing.border.*;

import org.amse.mARICa.Logger;
import org.amse.mARICa.model.impl.Settings;


public class MainFrame extends JFrame{
	private static final long serialVersionUID = 1L;
	
	//  package local -      
	static BoardPanel myBoardPanel;
	ArrowPanel myArrowPanel;
	ButtonPanel myButtonPanel;
	static Settings mySets;

	
	/**
	 *      Settings
	 * Game      "GO"
	 */
	public MainFrame(){
		super("Bantumi / ");
		setDefaultCloseOperation(EXIT_ON_CLOSE);
		setIconImage(new ImageIcon(MainFrame.class.getClassLoader().getResource("images/icon.png")).getImage()); 
		int iconPotWidth = 40;
		int iconPotHeight = 50;
		
		int countAllPots = Settings.getCountPots() * 2 + 2;
		myBoardPanel = new BoardPanel(countAllPots, iconPotWidth, iconPotHeight);
		myBoardPanel.setEnabled(false);		
		myArrowPanel = new ArrowPanel(myBoardPanel.getPreferredSize().height);
		myButtonPanel = new ButtonPanel();
		myButtonPanel.boardPanel = myBoardPanel;
		myButtonPanel.arrowPanel = myArrowPanel;
		myButtonPanel.mainFrame = this;
		myBoardPanel.buttonPanel = myButtonPanel;
		
		JPanel mainPanel = new JPanel();
		mainPanel.setBorder(new EmptyBorder(10, 5, 10, 5));
		mainPanel.setLayout(new BoxLayout(mainPanel, BoxLayout.X_AXIS));
		mainPanel.add(myArrowPanel);
		mainPanel.add(Box.createHorizontalGlue());
		mainPanel.add(myBoardPanel);
		mainPanel.add(Box.createHorizontalGlue());
		mainPanel.add(myButtonPanel);
				
		this.getContentPane().add(mainPanel);
		this.getLayeredPane().add(myBoardPanel.getHand(), Integer.valueOf(2));		
		this.addWindowListener(new WinAct());
		this.addKeyListener(new MyKeyListener());
		mainPanel.addKeyListener(new MyKeyListener());
	}
	
	private class WinAct extends WindowAdapter{
		public void windowClosing(WindowEvent e) {
			if (myButtonPanel.isGameNow){
				myButtonPanel.actionsButton();//  
			}
			MainFrame.this.dispose();			
		}
	}
	
	private class MyKeyListener implements KeyListener{

		public void keyPressed(KeyEvent key) {
			Logger.getInctanse().println("MainFrame Key");
			switch (key.getKeyCode()){
			case KeyEvent.VK_F1:
				createHelpWindow();
				break;
			case KeyEvent.VK_F3:
				createAboutWindow();
				break;
			}
		}
		public void keyReleased(KeyEvent e) {}
		public void keyTyped(KeyEvent e) {
			Logger.getInctanse().println("MainFrame Key");
		}		
	}
	
	private void createHelpWindow(){
		UIManager.put("OptionPane.yesButtonText", "OK (F1)");
		UIManager.put("OptionPane.noButtonText", " (F2)");
		UIManager.put("OptionPane.cancelButtonText", "  (F3)");
		JOptionPane.showMessageDialog(
				MainFrame.this, 
				new String[]{
						"Bantumi ",
						"\n",
						"\n"},
				"  Bantumi ",				
				JOptionPane.INFORMATION_MESSAGE);
		}
	
		private void createAboutWindow (){
			//     
			URL urlIconAbout = MainFrame.class.getClassLoader().getResource("images/iconAbout.jpg");
			JOptionPane.showMessageDialog(
					MainFrame.this,
					new String[]{"  -   ","Bantumi  v1.0"},
					" ",
					JOptionPane.INFORMATION_MESSAGE,
					new ImageIcon(urlIconAbout) );
		}

}
